@@ -42,7 +42,7 @@ typedef struct
 	char		*application;
 	char		*reason;
 	char		*connection;
-	int		 cookie;
+	guint32		 cookie;
 } GpmInhibitData;
 
 struct GpmInhibitPrivate
@@ -53,20 +53,84 @@ struct GpmInhibitPrivate
 
 G_DEFINE_TYPE (GpmInhibit, gpm_inhibit, G_TYPE_OBJECT)
 
+/**
+ * gpm_inhibit_cookie_compare_func
+ * @a: Pointer to the data to test
+ * @b: Pointer to a cookie to compare
+ *
+ * A GCompareFunc for comparing a cookie to a list.
+ *
+ * Return value: 0 if cookie matches
+ **/
+static gint
+gpm_inhibit_cookie_compare_func (gconstpointer a, gconstpointer b)
+{
+	GpmInhibitData *data;
+	guint32		cookie;
+	data = (GpmInhibitData*) a;
+	cookie = *((guint32*) b);
+	if (cookie == data->cookie)
+		return 0;
+	return 1;
+}
+
+/**
+ * gpm_inhibit_find_cookie:
+ * @inhibit: This inhibit instance
+ * @cookie: The cookie we are looking for
+ *
+ * Finds the data in the cookie list.
+ *
+ * Return value: The cookie data, or NULL if not found
+ **/
+static GpmInhibitData *
+gpm_inhibit_find_cookie (GpmInhibit *inhibit, guint32 cookie)
+{
+	GpmInhibitData *data;
+	GSList	       *ret;
+	ret = g_slist_find_custom (inhibit->priv->list, &cookie,
+				   gpm_inhibit_cookie_compare_func);
+	if (! ret) {
+		return NULL;
+	}
+	data = (GpmInhibitData *) ret->data;
+	return data;
+}
+
+/**
+ * gpm_inhibit_generate_cookie:
+ * @inhibit: This inhibit instance
+ *
+ * Returns a random cookie not already allocated.
+ *
+ * Return value: a new random cookie.
+ **/
+static guint32
+gpm_inhibit_generate_cookie (GpmInhibit *inhibit)
+{
+	guint32		cookie;
+
+	/* Iterate until we have a unique cookie */
+	do {
+		cookie = (guint32) g_random_int_range (1, G_MAXINT32);
+	} while (gpm_inhibit_find_cookie (inhibit, cookie));
+	return cookie;
+}
+
 /**
  * gpm_inhibit_add:
- * @connection:		Connection name, e.g. ":0.13"
+ * @connection: Connection name, e.g. ":0.13"
  * @application:	Application name, e.g. "Nautilus"
- * @reason:		Reason for inhibiting, e.g. "Copying files"
- * 
+ * @reason: Reason for inhibiting, e.g. "Copying files"
+ *
  * Allocates a random cookie used to identify the connection, as multiple
  * inhibit requests can come from one caller sharing a dbus connection.
  * We need to refcount internally, and data is saved in the GpmInhibitData
  * struct.
- * 
+ *
  * Return value: a new random cookie.
  **/
-int
+guint32
 gpm_inhibit_add (GpmInhibit *inhibit,
 		 const char *connection,
 		 const char *application,
@@ -76,22 +140,22 @@ gpm_inhibit_add (GpmInhibit *inhibit,
 	if (connection == NULL ||
 	    application == NULL ||
 	    reason == NULL) {
-		gpm_warning ("Recieved InhibitInactiveSleep, but application "
+		gpm_warning ("Recieved Inhibit, but application "
 			     "did not set the parameters correctly");
 		return -1;
 	}
 
 	/* seems okay, add to list */
 	GpmInhibitData *data = g_new (GpmInhibitData, 1);
-	data->cookie = g_random_int_range (0, 10240);
+	data->cookie = gpm_inhibit_generate_cookie (inhibit);
 	data->application = g_strdup (application);
 	data->connection = g_strdup (connection);
 	data->reason = g_strdup (reason);
 
-	inhibit->priv->list = g_slist_append (inhibit->priv->list, 
-                                              (gpointer) data);
+	inhibit->priv->list = g_slist_append (inhibit->priv->list,
+					      (gpointer) data);
 
-	gpm_debug ("Recieved InhibitInactiveSleep from '%s' (%s) because '%s' saving as #%i",
+	gpm_debug ("Recieved Inhibit from '%s' (%s) because '%s' saving as #%i",
 		   data->application, data->connection, data->reason, data->cookie);
 	return data->cookie;
 }
@@ -108,48 +172,39 @@ gpm_inhibit_free_data_object (GpmInhibitData *data)
 
 /**
  * gpm_inhibit_remove:
- * @connection:		Connection name
+ * @connection: Connection name
  * @application:	Application name
- * @cookie:		The cookie that we used to register
- * 
+ * @cookie: The cookie that we used to register
+ *
  * Removes a cookie and associated data from the GpmInhibitData struct.
  **/
 void
 gpm_inhibit_remove (GpmInhibit *inhibit,
 		    const char *connection,
-		    int		cookie)
+		    guint32	cookie)
 {
-	int a;
 	GpmInhibitData *data;
-	gboolean found = FALSE;
 
 	/* Only remove the correct cookie */
-	for (a=0; a<g_slist_length (inhibit->priv->list); a++) {
-		data = (GpmInhibitData *) g_slist_nth_data (inhibit->priv->list, a);
-		if (data->cookie == cookie) {
-			found = TRUE;
-			gpm_debug ("AllowInactiveSleep okay on '%s' as #%i",
-				   connection, cookie);
-			gpm_inhibit_free_data_object (data);
-			inhibit->priv->list = g_slist_remove (inhibit->priv->list,
-							      (gconstpointer) data);
-		}
-	}
-
-	/* If not found, then the developer never used InhibitInactiveSleep, and
-	   so put a warning out on the console */
-	if (!found) {
+	data = gpm_inhibit_find_cookie (inhibit, cookie);
+	if (data) {
+		gpm_debug ("UnInhibit okay on '%s' as #%i",
+			   connection, cookie);
+		gpm_inhibit_free_data_object (data);
+		inhibit->priv->list = g_slist_remove (inhibit->priv->list,
+						      (gconstpointer) data);
+	} else {
 		gpm_warning ("Cannot find registered program for #%i, so "
-			     "cannot do AllowInactiveSleep", cookie);
+			     "cannot do UnInhibit", cookie);
 	}
 }
 
 /**
  * gpm_inhibit_remove_dbus:
- * @connection:		Connection name
+ * @connection: Connection name
  * @application:	Application name
- * @cookie:		The cookie that we used to register
- * 
+ * @cookie: The cookie that we used to register
+ *
  * Checks to see if the dbus closed session is registered, in which case
  * unregister it.
  **/
@@ -197,7 +252,7 @@ dbus_name_owner_changed_session_cb (GpmDbusSessionMonitor *dbus_monitor,
 
 /**
  * gpm_inhibit_check:
- * 
+ *
  * Checks to see if we are being stopped from performing an action.
  *
  * Return value: TRUE if the action is OK, i.e. we have *not* been inhibited
@@ -224,7 +279,7 @@ gpm_inhibit_check (GpmInhibit *inhibit)
  **/
 void
 gpm_inhibit_get_message (GpmInhibit *inhibit,
-			 GString *message,
+			 GString    *message,
 			 const char *action)
 {
 	int a;
@@ -233,8 +288,8 @@ gpm_inhibit_get_message (GpmInhibit *inhibit,
 	if (g_slist_length (inhibit->priv->list) == 1) {
 		data = (GpmInhibitData *) g_slist_nth_data (inhibit->priv->list, 0);
 		g_string_append_printf (message, "<b>%s</b> has stopped the "
-				        "%s from taking place because <i>%s</i>.",
-				        data->application, action, data->reason);
+					"%s from taking place because <i>%s</i>.",
+					data->application, action, data->reason);
 	} else {
 		g_string_append_printf (message, "Multiple applications have stopped the "
 					"%s from taking place:", action);
