@@ -47,6 +47,8 @@ static void     gpm_power_finalize   (GObject       *object);
 
 #define GPM_POWER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GPM_TYPE_POWER, GpmPowerPrivate))
 
+#define GPM_POWER_MIN_CHARGED_PERCENTAGE	90
+
 struct GpmPowerPrivate
 {
 	gboolean		 on_ac;
@@ -124,7 +126,7 @@ gpm_power_battery_is_charged (GpmPowerStatus *status)
 {
 	if (! status->is_charging &&
 	    ! status->is_discharging &&
-	    status->percentage_charge > 90) {
+	    status->percentage_charge > GPM_POWER_MIN_CHARGED_PERCENTAGE) {
 		return TRUE;
 	}
 	return FALSE;
@@ -245,6 +247,11 @@ battery_device_cache_entry_update_all (GpmPowerDevice *entry)
 				float capacity;
 				capacity = 100.0f / (float) status->design_charge;
 				status->capacity = capacity * (float) status->last_full_charge;
+				if (status->capacity > 100) {
+					gpm_debug ("rounding down capactity from "
+						   "%i to 100", status->capacity);
+					status->capacity = 100;
+				}
 			}
 		}
 	}
@@ -286,7 +293,7 @@ gpm_power_exp_aver (int previous, int new, int factor_pc)
 	float factor_inv = 1;
 	if (previous == 0 || factor_pc == 0) {
 		/* startup, or re-initialization - we have no data */
-		gpm_debug ("Telling rate with no ave factor!");
+		gpm_debug ("Telling rate with no ave factor (okay once)");
 		result = new;
 	} else {
 		factor = (float) factor_pc / 100.0f;
@@ -622,9 +629,9 @@ get_power_unit_suffix (GpmPowerUnit unit)
 }
 
 GpmPowerDevice *
-gpm_power_get_battery_device_entry (GpmPower		*power,
+gpm_power_get_battery_device_entry (GpmPower	 *power,
 				    GpmPowerKind  battery_kind,
-				    gint		 device_num)
+				    gint	  device_num)
 {
 	const char		*udi;
 	GpmPowerDevice *device;
@@ -981,6 +988,22 @@ battery_kind_cache_update (GpmPower		 *power,
 		type_status->is_charging = FALSE;
 	}
 
+	/* If the primary battery is neither charging nor discharging, and
+	 * the charge is low the battery is most likely broken.
+	 * In this case, we'll use the ac_adaptor to determine whether it's
+	 * charging or not. */
+	if (entry->battery_kind == GPM_POWER_KIND_PRIMARY &&
+	    (! type_status->is_charging) && (! type_status->is_discharging) &&
+	    type_status->percentage_charge > 0 &&
+	    type_status->percentage_charge < GPM_POWER_MIN_CHARGED_PERCENTAGE) {
+		gboolean on_ac;
+		on_ac = gpm_hal_monitor_get_on_ac (power->priv->hal_monitor);
+		gpm_debug ("Battery is neither charging nor discharging, "
+			   "using ac_adaptor value %i", on_ac);
+		type_status->is_charging = on_ac;
+		type_status->is_discharging = !(on_ac);
+	}
+
 	gpm_debug ("%i devices of type %s", num_present, gpm_power_kind_to_localised_string (entry->battery_kind));
 
 	/* Perform following calculations with floating point otherwise we might
@@ -1205,26 +1228,26 @@ power_get_summary_for_battery_kind (GpmPower		*power,
 
 	if (gpm_power_battery_is_charged (status)) {
 
-			g_string_append_printf (summary, "%s", _("fully charged"));
+			g_string_append (summary, _("fully charged"));
 
 	} else if (status->is_discharging) {
 
 		if (status->remaining_time > 60) {
-			g_string_append_printf (summary, "%s %s",
-						timestring, _("remaining"));
+			g_string_append_printf (summary, _("%s remaining"),
+						timestring);
 		} else {
 			/* don't display "Unknown remaining" */
-			g_string_append_printf (summary, "%s", _("discharging"));
+			g_string_append (summary, _("discharging"));
 		}
 
 	} else if (status->is_charging || power->priv->on_ac) {
 
 		if (status->remaining_time > 60) {
-			g_string_append_printf (summary, "%s %s",
-						timestring, _("until charged"));
+			g_string_append_printf (summary, _("%s until charged"),
+						timestring);
 		} else {
 			/* don't display "Unknown remaining" */
-			g_string_append_printf (summary, "%s", _("charging"));
+			g_string_append (summary, _("charging"));
 		}
 
 	} else {
