@@ -24,6 +24,8 @@
 #  include <config.h>
 #endif
 
+#define USE_EGGTRAYICON		(GTK_MINOR_VERSION < 9)
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <time.h>
@@ -39,9 +41,13 @@
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 
+#if ! USE_EGGTRAYICON
+#include <gtk/gtkstatusicon.h>
+#endif
+
 #include <libgnomeui/gnome-help.h> /* for gnome_help_display */
 
-#if LIBNOTIFY_VERSION_MINOR >= 2
+#ifdef HAVE_LIBNOTIFY
 #include <libnotify/notify.h>
 #endif
 
@@ -52,28 +58,31 @@
 
 static void     gpm_tray_icon_class_init (GpmTrayIconClass *klass);
 static void     gpm_tray_icon_init       (GpmTrayIcon      *tray_icon);
-static void     gpm_tray_icon_finalize   (GObject          *object);
+static void     gpm_tray_icon_finalize   (GObject	   *object);
 
 #define GPM_TRAY_ICON_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GPM_TYPE_TRAY_ICON, GpmTrayIconPrivate))
 
 struct GpmTrayIconPrivate
 {
-	GtkUIManager   *ui_manager;
-	GtkActionGroup *actiongroup;
-	GtkTooltips    *tooltips;
-	GtkWidget      *popup_menu;
-	GtkWidget      *image;
-	GtkWidget      *ebox;
+	GtkUIManager	*ui_manager;
+	GtkActionGroup	*actiongroup;
+	GtkWidget	*popup_menu;
+#if USE_EGGTRAYICON
+	GtkTooltips	*tooltips;
+	GtkWidget	*image;
+	GtkWidget	*ebox;
+#else
+	GtkStatusIcon	*status_icon;
+#endif
 
-	gboolean        show_notifications;
-	gboolean        is_visible;
+	gboolean	 show_notifications;
+	gboolean	 is_visible;
 
-	gboolean        can_suspend;
-	gboolean        can_hibernate;
+	gboolean	 can_suspend;
+	gboolean	 can_hibernate;
 
-#if (LIBNOTIFY_VERSION_MINOR == 2)
-	NotifyHandle   *notify;
-#elif (LIBNOTIFY_VERSION_MINOR >= 3)
+	char		*stock_id;
+#ifdef HAVE_LIBNOTIFY
 	NotifyNotification *notify;
 #endif
 };
@@ -81,6 +90,7 @@ struct GpmTrayIconPrivate
 enum {
 	SUSPEND,
 	HIBERNATE,
+	SHOW_INFO,
 	LAST_SIGNAL
 };
 
@@ -89,26 +99,23 @@ enum {
 	PROP_MODE
 };
 
-typedef enum {
-	LIBNOTIFY_URGENCY_CRITICAL = 1,
-	LIBNOTIFY_URGENCY_NORMAL   = 2,
-	LIBNOTIFY_URGENCY_LOW      = 3
-} LibNotifyEventType;
-
 static void gpm_tray_icon_suspend_cb		(GtkAction *action, GpmTrayIcon *icon);
 static void gpm_tray_icon_hibernate_cb		(GtkAction *action, GpmTrayIcon *icon);
+static void gpm_tray_icon_show_info_cb	 	(GtkAction *action, GpmTrayIcon *icon);
 static void gpm_tray_icon_show_preferences_cb	(GtkAction *action, GpmTrayIcon *icon);
 static void gpm_tray_icon_show_help_cb		(GtkAction *action, GpmTrayIcon *icon);
 static void gpm_tray_icon_show_about_cb		(GtkAction *action, GpmTrayIcon *icon);
 
 static GtkActionEntry gpm_tray_icon_action_entries [] =
 {
-	{ "TraySuspend", GPM_STOCK_SUSPEND_TO_RAM, N_("_Suspend"),
+	{ "TraySuspend", GPM_STOCK_SUSPEND, N_("_Suspend"),
 	  NULL, N_("Suspend the computer"), G_CALLBACK (gpm_tray_icon_suspend_cb) },
-	{ "TrayHibernate", GPM_STOCK_SUSPEND_TO_DISK, N_("Hi_bernate"),
+	{ "TrayHibernate", GPM_STOCK_HIBERNATE, N_("Hi_bernate"),
 	  NULL, N_("Make the computer go to sleep"), G_CALLBACK (gpm_tray_icon_hibernate_cb) },
 	{ "TrayPreferences", GTK_STOCK_PREFERENCES, N_("_Preferences"),
 	  NULL, NULL, G_CALLBACK (gpm_tray_icon_show_preferences_cb) },
+	{ "TrayInfo", GTK_STOCK_DIALOG_INFO, N_("_Information"),
+	  NULL, NULL, G_CALLBACK (gpm_tray_icon_show_info_cb) },
 	{ "TrayHelp", GTK_STOCK_HELP, N_("_Help"), NULL,
 	  NULL, G_CALLBACK (gpm_tray_icon_show_help_cb) },
 	{ "TrayAbout", GTK_STOCK_ABOUT, N_("_About"), NULL,
@@ -116,11 +123,15 @@ static GtkActionEntry gpm_tray_icon_action_entries [] =
 };
 static guint gpm_tray_icon_n_action_entries = G_N_ELEMENTS (gpm_tray_icon_action_entries);
 
-static GObjectClass *parent_class = NULL;
-static guint         signals [LAST_SIGNAL] = { 0, };
+static guint	 signals [LAST_SIGNAL] = { 0, };
 
 G_DEFINE_TYPE (GpmTrayIcon, gpm_tray_icon, EGG_TYPE_TRAY_ICON)
 
+/**
+ * gpm_tray_icon_enable_suspend:
+ * @icon: This TrayIcon class instance
+ * @enabled: If we should enable (i.e. show) the suspend icon
+ **/
 void
 gpm_tray_icon_enable_suspend (GpmTrayIcon *icon,
 			      gboolean     enabled)
@@ -131,7 +142,6 @@ gpm_tray_icon_enable_suspend (GpmTrayIcon *icon,
 		GtkAction *action;
 
 		icon->priv->can_suspend = enabled;
-
 		action = gtk_action_group_get_action (icon->priv->actiongroup,
 						      "TraySuspend");
 		gtk_action_set_visible (GTK_ACTION (action),
@@ -139,6 +149,11 @@ gpm_tray_icon_enable_suspend (GpmTrayIcon *icon,
 	}
 }
 
+/**
+ * gpm_tray_icon_enable_hibernate:
+ * @icon: This TrayIcon class instance
+ * @enabled: If we should enable (i.e. show) the hibernate icon
+ **/
 void
 gpm_tray_icon_enable_hibernate (GpmTrayIcon *icon,
 				gboolean     enabled)
@@ -149,7 +164,6 @@ gpm_tray_icon_enable_hibernate (GpmTrayIcon *icon,
 		GtkAction *action;
 
 		icon->priv->can_hibernate = enabled;
-
 		action = gtk_action_group_get_action (icon->priv->actiongroup,
 						      "TrayHibernate");
 		gtk_action_set_visible (GTK_ACTION (action),
@@ -158,6 +172,11 @@ gpm_tray_icon_enable_hibernate (GpmTrayIcon *icon,
 	}
 }
 
+/**
+ * gpm_tray_icon_set_tooltip:
+ * @icon: This TrayIcon class instance
+ * @tooltip: The tooltip text, e.g. "Batteries fully charged"
+ **/
 void
 gpm_tray_icon_set_tooltip (GpmTrayIcon *icon,
 			   const char  *tooltip)
@@ -165,33 +184,76 @@ gpm_tray_icon_set_tooltip (GpmTrayIcon *icon,
 	g_return_if_fail (GPM_IS_TRAY_ICON (icon));
 	g_return_if_fail (tooltip != NULL);
 
+#if USE_EGGTRAYICON
 	gtk_tooltips_set_tip (icon->priv->tooltips,
 			      GTK_WIDGET (icon),
 			      tooltip, NULL);
+#else
+	gtk_status_icon_set_tooltip (GTK_STATUS_ICON (icon->priv->status_icon),
+				     tooltip);
+#endif
 }
 
+/**
+ * gpm_tray_icon_set_image_from_stock:
+ * @icon: This TrayIcon class instance
+ * @stock_id: The icon name, e.g. GPM_STOCK_APP_ICON, or NULL to remove.
+ *
+ * Loads a pixmap from disk, and sets as the tooltip icon
+ **/
 void
 gpm_tray_icon_set_image_from_stock (GpmTrayIcon *icon,
 				    const char  *stock_id)
 {
 	g_return_if_fail (GPM_IS_TRAY_ICON (icon));
-	gpm_debug ("Setting icon to %s", stock_id);
 
 	if (stock_id) {
-		gtk_image_set_from_stock (GTK_IMAGE (icon->priv->image),
-					  stock_id,
-					  GTK_ICON_SIZE_LARGE_TOOLBAR);
+		/* we only set a new icon if the name differs */
+		if (strcmp (icon->priv->stock_id, stock_id) != 0) {
+			gpm_debug ("Setting icon to %s", stock_id);
+#if USE_EGGTRAYICON
+			gtk_image_set_from_icon_name (GTK_IMAGE (icon->priv->image),
+						      stock_id,
+						      GTK_ICON_SIZE_LARGE_TOOLBAR);
+#else
+			gtk_status_icon_set_from_icon_name (GTK_STATUS_ICON (icon->priv->status_icon), stock_id);
+			gtk_status_icon_set_visible (GTK_STATUS_ICON (icon->priv->status_icon), TRUE);
+#endif
+			/* don't keep trying to set the same icon */
+		        g_free (icon->priv->stock_id);
+			icon->priv->stock_id = g_strdup (stock_id);
+		}
 	} else {
-		/* FIXME: gtk_image_clear requires gtk 2.8, so until we
-		 * depend on more then 2.6 (required for FC4) we have to
-		 * comment it out
-		gtk_image_clear (GTK_IMAGE (icon->priv->image));*/
+		/* get rid of the icon */
+#if USE_EGGTRAYICON
+		gtk_image_clear (GTK_IMAGE (icon->priv->image));
 		if (GTK_WIDGET_VISIBLE (icon->priv->image)) {
 			gtk_widget_queue_resize (GTK_WIDGET (icon->priv->image));
 		}
+#else
+		gtk_status_icon_set_visible (GTK_STATUS_ICON (icon->priv->status_icon), FALSE);
+#endif
 	}
 }
 
+/**
+ * gpm_tray_icon_show_info_cb:
+ * @action: A valid GtkAction
+ * @icon: This TrayIcon class instance
+ **/
+static void
+gpm_tray_icon_show_info_cb (GtkAction   *action,
+			    GpmTrayIcon *icon)
+{
+	gpm_debug ("emitting show_info");
+	g_signal_emit (icon, signals [SHOW_INFO], 0);
+}
+
+/**
+ * gpm_tray_icon_hibernate_cb:
+ * @action: A valid GtkAction
+ * @icon: This TrayIcon class instance
+ **/
 static void
 gpm_tray_icon_hibernate_cb (GtkAction   *action,
 			    GpmTrayIcon *icon)
@@ -200,6 +262,11 @@ gpm_tray_icon_hibernate_cb (GtkAction   *action,
 	g_signal_emit (icon, signals [HIBERNATE], 0);
 }
 
+/**
+ * gpm_tray_icon_suspend_cb:
+ * @action: A valid GtkAction
+ * @icon: This TrayIcon class instance
+ **/
 static void
 gpm_tray_icon_suspend_cb (GtkAction   *action,
 			  GpmTrayIcon *icon)
@@ -208,6 +275,11 @@ gpm_tray_icon_suspend_cb (GtkAction   *action,
 	g_signal_emit (icon, signals [SUSPEND], 0);
 }
 
+/**
+ * gpm_tray_icon_show_preferences_cb:
+ * @action: A valid GtkAction
+ * @icon: This TrayIcon class instance
+ **/
 static void
 gpm_tray_icon_show_preferences_cb (GtkAction   *action,
 				   GpmTrayIcon *icon)
@@ -219,6 +291,11 @@ gpm_tray_icon_show_preferences_cb (GtkAction   *action,
 	}
 }
 
+/**
+ * gpm_tray_icon_show_help_cb:
+ * @action: A valid GtkAction
+ * @icon: This TrayIcon class instance
+ **/
 static void
 gpm_tray_icon_show_help_cb (GtkAction   *action,
 			    GpmTrayIcon *icon)
@@ -232,8 +309,13 @@ gpm_tray_icon_show_help_cb (GtkAction   *action,
 	}
 }
 
+/**
+ * gpm_tray_icon_show_about_cb:
+ * @action: A valid GtkAction
+ * @icon: This TrayIcon class instance
+ **/
 static void
-gpm_tray_icon_show_about_cb (GtkAction  *action,
+gpm_tray_icon_show_about_cb (GtkAction   *action,
 			     GpmTrayIcon *icon)
 {
 	const char *authors[] = {
@@ -246,6 +328,7 @@ gpm_tray_icon_show_about_cb (GtkAction  *action,
 		NULL};
 	const char *artists[] = {
 		"Diana Fong <dfong@redhat.com>",
+		"Jakub Steiner <jimmac@ximian.com>",
 		NULL};
 	const char *license[] = {
 		N_("Licensed under the GNU General Public License Version 2"),
@@ -263,40 +346,38 @@ gpm_tray_icon_show_about_cb (GtkAction  *action,
 		   "02110-1301, USA.")
 	};
   	const char  *translators = _("translator-credits");
-	char        *license_trans;
-	GdkPixbuf   *pixbuf;
+	char	    *license_trans;
 
-	/*
-	 * Translators comment: put your own name here to appear in the about dialog.
-	 */
+	/* Translators comment: put your own name here to appear in the about dialog. */
   	if (!strcmp (translators, "translator-credits")) {
 		translators = NULL;
 	}
 
-	pixbuf = gdk_pixbuf_new_from_file (GPM_DATA "gnome-power.png", NULL);
-
 	license_trans = g_strconcat (_(license[0]), "\n\n", _(license[1]), "\n\n",
 				     _(license[2]), "\n\n", _(license[3]), "\n",  NULL);
 
-	gtk_window_set_default_icon_from_file (GPM_DATA "gnome-power-manager.png", NULL);
-
+	gtk_window_set_default_icon_name (GPM_STOCK_APP_ICON);
 	gtk_show_about_dialog (NULL,
-		               "name", GPM_NAME,
-		               "version", VERSION,
-		               "copyright", "Copyright \xc2\xa9 2005 Richard Hughes",
-		               "license", license_trans,
-		               "website", GPM_HOMEPAGE_URL,
-		               "comments", GPM_DESCRIPTION,
-		               "authors", authors,
-		               "documenters", documenters,
-		               "artists", artists,
-		               "translator-credits", translators,
-		               "logo", pixbuf,
-		               NULL);
-
-	g_object_unref (pixbuf);
+			       "name", GPM_NAME,
+			       "version", VERSION,
+			       "copyright", "Copyright \xc2\xa9 2005-2006 Richard Hughes",
+			       "license", license_trans,
+			       "website", GPM_HOMEPAGE_URL,
+			       "comments", GPM_DESCRIPTION,
+			       "authors", authors,
+			       "documenters", documenters,
+			       "artists", artists,
+			       "translator-credits", translators,
+			       "logo-icon-name", GPM_STOCK_APP_ICON,
+			       NULL);
 }
 
+#if USE_EGGTRAYICON
+/**
+ * tray_popup_position_menu:
+ *
+ * Popup the drop-down menu at the base of the icon
+ **/
 static void
 tray_popup_position_menu (GtkMenu  *menu,
 			  int      *x,
@@ -306,8 +387,8 @@ tray_popup_position_menu (GtkMenu  *menu,
 {
 	GtkWidget     *widget;
 	GtkRequisition requisition;
-	gint           menu_xpos;
-	gint           menu_ypos;
+	gint	       menu_xpos;
+	gint	       menu_ypos;
 
 	widget = GTK_WIDGET (user_data);
 
@@ -328,15 +409,36 @@ tray_popup_position_menu (GtkMenu  *menu,
 	*y = menu_ypos;
 	*push_in = TRUE;
 }
+#endif
 
+/**
+ * gpm_tray_icon_popup_cleared_cd:
+ * @widget: The popup Gtkwidget
+ * @icon: This TrayIcon class instance
+ *
+ * We have to re-enable the tooltip when the popup is removed
+ **/
 static void
-gpm_tray_icon_popup_cleared_cd (GtkWidget   *mo,
+gpm_tray_icon_popup_cleared_cd (GtkWidget   *widget,
 				GpmTrayIcon *icon)
 {
+	g_return_if_fail (GPM_IS_TRAY_ICON (icon));
+
+#if USE_EGGTRAYICON
 	/* we enable the tooltip as the menu has gone */
 	gtk_tooltips_enable (icon->priv->tooltips);
+#endif
 }
 
+#if USE_EGGTRAYICON
+/**
+ * gpm_tray_icon_button_press_cb:
+ * @widget: The tray icon widget
+ * @event: Valid event
+ * @icon: This TrayIcon class instance
+ *
+ * What do do when the button is left, right, middle clicked etc.
+ **/
 static gboolean
 gpm_tray_icon_button_press_cb (GtkWidget      *widget,
 			       GdkEventButton *event,
@@ -351,66 +453,50 @@ gpm_tray_icon_button_press_cb (GtkWidget      *widget,
 
 	/* we disable the tooltip so it doesn't clash with the menu. See #331075 */
 	gtk_tooltips_disable (icon->priv->tooltips);
-
 	gtk_menu_popup (GTK_MENU (popup), NULL, NULL,
 			tray_popup_position_menu, widget,
 			2,
 			gtk_get_current_event_time ());
 	return TRUE;
 }
+#endif
 
+/**
+ * gpm_tray_icon_sync_actions:
+ * @icon: This TrayIcon class instance
+ *
+ * Syncs the private icon->priv->can_* variables with the icon states
+ **/
 static void
-gpm_tray_icon_set_property (GObject		  *object,
-			       guint		   prop_id,
-			       const GValue	  *value,
-			       GParamSpec	  *pspec)
-{
-	switch (prop_id) {
-	default:
-		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-		break;
-	}
-}
-
-static void
-gpm_tray_icon_get_property (GObject		  *object,
-			       guint		   prop_id,
-			       GValue		  *value,
-			       GParamSpec	  *pspec)
-{
-	switch (prop_id) {
-	default:
-		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-		break;
-	}
-}
-
-static void
-gpm_tray_icon_sync_actions (GpmTrayIcon *tray)
+gpm_tray_icon_sync_actions (GpmTrayIcon *icon)
 {
 	GtkAction *action;
+	g_return_if_fail (GPM_IS_TRAY_ICON (icon));
 
-	if (tray->priv->actiongroup != NULL) {
-		action = gtk_action_group_get_action (tray->priv->actiongroup,
+	if (icon->priv->actiongroup != NULL) {
+		action = gtk_action_group_get_action (icon->priv->actiongroup,
 						      "TraySuspend");
+		gtk_action_set_visible (GTK_ACTION (action), icon->priv->can_suspend);
 
-		gtk_action_set_visible (GTK_ACTION (action), tray->priv->can_suspend);
-
-		action = gtk_action_group_get_action (tray->priv->actiongroup,
+		action = gtk_action_group_get_action (icon->priv->actiongroup,
 						      "TrayHibernate");
-
-		gtk_action_set_visible (GTK_ACTION (action), tray->priv->can_hibernate);
+		gtk_action_set_visible (GTK_ACTION (action), icon->priv->can_hibernate);
 	}
 }
 
+/**
+ * gpm_tray_icon_constructor:
+ *
+ * Connects the UI to the tray icon instance
+ **/
 static GObject *
-gpm_tray_icon_constructor (GType                  type,
-			   guint                  n_construct_properties,
+gpm_tray_icon_constructor (GType		  type,
+			   guint		  n_construct_properties,
 			   GObjectConstructParam *construct_properties)
 {
 	GpmTrayIcon      *tray;
 	GpmTrayIconClass *klass;
-	GError           *error = NULL;
+	GError		 *error = NULL;
 	GtkWidget	 *widget;
 
 	klass = GPM_TRAY_ICON_CLASS (g_type_class_peek (GPM_TYPE_TRAY_ICON));
@@ -438,6 +524,7 @@ gpm_tray_icon_constructor (GType                  type,
 					   "    <menuitem action=\"TrayHibernate\" />"
 					   "    <separator />"
 					   "    <menuitem action=\"TrayPreferences\" />"
+					   "    <menuitem action=\"TrayInfo\" />"
 					   "    <separator />"
 					   "    <menuitem action=\"TrayHelp\" />"
 					   "    <menuitem action=\"TrayAbout\" />"
@@ -460,55 +547,125 @@ gpm_tray_icon_constructor (GType                  type,
 	return G_OBJECT (tray);
 }
 
+/**
+ * gpm_tray_icon_class_init:
+ **/
 static void
 gpm_tray_icon_class_init (GpmTrayIconClass *klass)
 {
 	GObjectClass   *object_class = G_OBJECT_CLASS (klass);
 
-	parent_class = g_type_class_peek_parent (klass);
-
 	object_class->finalize	   = gpm_tray_icon_finalize;
-	object_class->get_property = gpm_tray_icon_get_property;
-	object_class->set_property = gpm_tray_icon_set_property;
 	object_class->constructor  = gpm_tray_icon_constructor;
 
 	g_type_class_add_private (klass, sizeof (GpmTrayIconPrivate));
 
 	signals [SUSPEND] =
 		g_signal_new ("suspend",
-		              G_TYPE_FROM_CLASS (object_class),
-		              G_SIGNAL_RUN_LAST,
-		              G_STRUCT_OFFSET (GpmTrayIconClass, suspend),
-		              NULL,
-		              NULL,
-		              g_cclosure_marshal_VOID__VOID,
-		              G_TYPE_NONE,
-		              0);
+			      G_TYPE_FROM_CLASS (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (GpmTrayIconClass, suspend),
+			      NULL,
+			      NULL,
+			      g_cclosure_marshal_VOID__VOID,
+			      G_TYPE_NONE,
+			      0);
 	signals [HIBERNATE] =
 		g_signal_new ("hibernate",
-		              G_TYPE_FROM_CLASS (object_class),
-		              G_SIGNAL_RUN_LAST,
-		              G_STRUCT_OFFSET (GpmTrayIconClass, hibernate),
-		              NULL,
-		              NULL,
-		              g_cclosure_marshal_VOID__VOID,
-		              G_TYPE_NONE,
-		              0);
+			      G_TYPE_FROM_CLASS (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (GpmTrayIconClass, hibernate),
+			      NULL,
+			      NULL,
+			      g_cclosure_marshal_VOID__VOID,
+			      G_TYPE_NONE,
+			      0);
+	signals [SHOW_INFO] =
+		g_signal_new ("show-info",
+			      G_TYPE_FROM_CLASS (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (GpmTrayIconClass, hibernate),
+			      NULL,
+			      NULL,
+			      g_cclosure_marshal_VOID__VOID,
+			      G_TYPE_NONE,
+			      0);
 }
 
+/**
+ * gpm_tray_icon_show:
+ * @icon: This TrayIcon class instance
+ * @enabled: If we should show the tray
+ **/
 void
 gpm_tray_icon_show (GpmTrayIcon *icon,
 		    gboolean     enabled)
 {
+	g_return_if_fail (GPM_IS_TRAY_ICON (icon));
+
 	if (enabled) {
+#if USE_EGGTRAYICON
 		gtk_widget_show_all (GTK_WIDGET (icon));
+#else
+		gtk_status_icon_set_visible (GTK_STATUS_ICON (icon->priv->status_icon), TRUE);
+#endif
 		icon->priv->is_visible = TRUE;
 	} else {
+#if USE_EGGTRAYICON
 		gtk_widget_hide_all (GTK_WIDGET (icon));
+#else
+		gtk_status_icon_set_visible (GTK_STATUS_ICON (icon->priv->status_icon), FALSE);
+#endif
 		icon->priv->is_visible = FALSE;
 	}
 }
 
+#if !USE_EGGTRAYICON
+/**
+ * gpm_tray_icon_popup_menu_cb:
+ * @button: Which buttons are pressed
+ * @icon: This TrayIcon class instance
+ *
+ * Display the popup menu.
+ **/
+static void
+gpm_tray_icon_popup_menu_cb (GtkStatusIcon *status_icon,
+			     guint          button,
+			     guint32        activate_time,
+			     GpmTrayIcon    *icon)
+{
+	GtkWidget *popup;
+	popup = gtk_ui_manager_get_widget (GTK_UI_MANAGER (icon->priv->ui_manager),
+					   "/GpmTrayPopup");
+	gtk_menu_set_screen (GTK_MENU (popup),
+			     gtk_widget_get_screen (GTK_WIDGET (icon)));
+
+	gtk_menu_popup (GTK_MENU (popup), NULL, NULL,
+			gtk_status_icon_position_menu, status_icon,
+			button, activate_time);
+}
+
+/**
+ * gpm_tray_icon_activate_cb:
+ * @button: Which buttons are pressed
+ * @icon: This TrayIcon class instance
+ *
+ * Callback when the icon is clicked
+ **/
+static void
+gpm_tray_icon_activate_cb (GtkStatusIcon *status_icon,
+			   GpmTrayIcon   *icon)
+{
+	g_debug ("icon activated");
+}
+#endif
+
+/**
+ * gpm_tray_icon_init:
+ * @icon: This TrayIcon class instance
+ *
+ * Initialise the tray object, and set up libnotify
+ **/
 static void
 gpm_tray_icon_init (GpmTrayIcon *icon)
 {
@@ -518,33 +675,48 @@ gpm_tray_icon_init (GpmTrayIcon *icon)
 
 	/* FIXME: make this a property */
 	icon->priv->show_notifications = TRUE;
+	icon->priv->stock_id = g_strdup ("about-blank");
 
-	icon->priv->tooltips = gtk_tooltips_new ();
 	icon->priv->ui_manager = gtk_ui_manager_new ();
 
+#if USE_EGGTRAYICON
+	icon->priv->tooltips = gtk_tooltips_new ();
 	icon->priv->ebox = gtk_event_box_new ();
 	g_signal_connect_object (G_OBJECT (icon->priv->ebox),
-		                 "button_press_event",
-		                 G_CALLBACK (gpm_tray_icon_button_press_cb),
-		                 icon, 0);
+				 "button_press_event",
+				 G_CALLBACK (gpm_tray_icon_button_press_cb),
+				 icon, 0);
 
 	icon->priv->image = gtk_image_new ();
-
 	gtk_container_add (GTK_CONTAINER (icon->priv->ebox), icon->priv->image);
 
 	gtk_container_add (GTK_CONTAINER (icon), icon->priv->ebox);
+#else
+	icon->priv->status_icon = gtk_status_icon_new ();
+	g_signal_connect_object (G_OBJECT (icon->priv->status_icon),
+				 "popup_menu",
+				 G_CALLBACK (gpm_tray_icon_popup_menu_cb),
+				 icon, 0);
+	g_signal_connect_object (G_OBJECT (icon->priv->status_icon),
+				 "activate",
+				 G_CALLBACK (gpm_tray_icon_activate_cb),
+				 icon, 0);
+#endif
+
 	gpm_tray_icon_show (GPM_TRAY_ICON (icon), FALSE);
 
-#if (LIBNOTIFY_VERSION_MINOR >= 3)
+#ifdef HAVE_LIBNOTIFY
 	ret = notify_init (GPM_NAME);
-#elif (LIBNOTIFY_VERSION_MINOR == 2)
-	ret = notify_glib_init (GPM_NAME, NULL);
 #endif
 	if (!ret) {
 		gpm_warning ("gpm_tray_icon_init failed");
 	}
 }
 
+/**
+ * gpm_tray_icon_finalize:
+ * @object: This TrayIcon class instance
+ **/
 static void
 gpm_tray_icon_finalize (GObject *object)
 {
@@ -557,146 +729,86 @@ gpm_tray_icon_finalize (GObject *object)
 
 	g_return_if_fail (tray_icon->priv != NULL);
 
+#if USE_EGGTRAYICON
 	gtk_object_destroy (GTK_OBJECT (tray_icon->priv->tooltips));
+#endif
 
-	G_OBJECT_CLASS (parent_class)->finalize (object);
+	G_OBJECT_CLASS (gpm_tray_icon_parent_class)->finalize (object);
 }
 
+/**
+ * gpm_tray_icon_new:
+ * Return value: A new TrayIcon object.
+ **/
 GpmTrayIcon *
 gpm_tray_icon_new (void)
 {
 	GpmTrayIcon *tray_icon;
-
 	tray_icon = g_object_new (GPM_TYPE_TRAY_ICON, NULL);
-
 	return GPM_TRAY_ICON (tray_icon);
 }
 
-#if LIBNOTIFY_VERSION_MINOR >= 2
-static gboolean
-get_widget_position (GtkWidget *widget,
-		     int       *x,
-		     int       *y)
-{
-	g_assert (widget);
-
-	gdk_window_get_origin (GDK_WINDOW (widget->window), x, y);
-
-	*x += widget->allocation.x;
-	*y += widget->allocation.y;
-	*x += widget->allocation.width / 2;
-	*y += widget->allocation.height;
-
-	gpm_debug ("widget position x=%i, y=%i", *x, *y);
-
-	return TRUE;
-}
-#endif
-
-#if (LIBNOTIFY_VERSION_MINOR >= 3)
+#ifdef HAVE_LIBNOTIFY
 
+/**
+ * notification_closed_cb:
+ * @notify: our libnotify instance
+ * @icon: This TrayIcon class instance
+ **/
 static void
 notification_closed_cb (NotifyNotification *notify,
-			GpmTrayIcon        *tray)
+			GpmTrayIcon	*icon)
 {
 	/* just invalidate the pointer */
 	gpm_debug ("caught notification closed signal");
-	tray->priv->notify = NULL;
+	icon->priv->notify = NULL;
 }
 
+/**
+ * libnotify_event:
+ * @icon: This icon class instance
+ * @title: The title, e.g. "Battery Low"
+ * @content: The contect, e.g. "17 minutes remaining"
+ * @timeout: The time we should remain on screen in seconds
+ * @msgicon: The icon to display, or NULL, e.g. GPM_STOCK_UPS_CHARGING_080
+ * @urgency: The urgency type, e.g. GPM_NOTIFY_URGENCY_CRITICAL
+ *
+ * Does a libnotify messagebox dialogue.
+ * Return value: success
+ **/
 static gboolean
-libnotify_event (GpmTrayIcon             *tray,
-		 guint                    timeout,	/* in seconds */
-		 const char              *subject,
-		 const char              *content,
-		 const LibNotifyEventType urgency)
+libnotify_event (GpmTrayIcon    *icon,
+		 const char	*title,
+		 const char	*content,
+		 guint		 timeout,
+		 const char	*msgicon,
+		 GpmNotifyLevel	 urgency)
 {
-	int x;
-	int y;
-
-	if (tray->priv->notify != NULL) {
-		notify_notification_close (tray->priv->notify, NULL);
+	if (icon->priv->notify != NULL) {
+		notify_notification_close (icon->priv->notify, NULL);
 	}
 
-	tray->priv->notify = notify_notification_new (subject, content,
-						      GPM_STOCK_BATTERY_DISCHARGING_100,
-						      NULL);
-
-	notify_notification_set_timeout (tray->priv->notify, timeout * 1000);
-
-	if (tray->priv->is_visible) {
-		get_widget_position (GTK_WIDGET (tray), &x, &y);
-		notify_notification_set_hint_int32 (tray->priv->notify, "x", x);
-		notify_notification_set_hint_int32 (tray->priv->notify, "y", y);
+	/* Point to the center of the icon as per the GNOME HIG, #338638 */
+	GtkWidget *point = NULL;
+#if USE_EGGTRAYICON
+	if (icon->priv->is_visible) {
+		point = icon->priv->image;
 	}
+#endif
+	icon->priv->notify = notify_notification_new (title, content,
+						      msgicon, point);
 
-	if (urgency == LIBNOTIFY_URGENCY_CRITICAL) {
+	notify_notification_set_timeout (icon->priv->notify, timeout * 1000);
+
+	if (urgency == GPM_NOTIFY_URGENCY_CRITICAL) {
 		gpm_warning ("libnotify: %s : %s", GPM_NAME, content);
 	} else {
 		gpm_debug ("libnotify: %s : %s", GPM_NAME, content);
 	}
 
-	g_signal_connect (tray->priv->notify, "closed", G_CALLBACK (notification_closed_cb), tray);
+	g_signal_connect (icon->priv->notify, "closed", G_CALLBACK (notification_closed_cb), icon);
 
-	if (! notify_notification_show (tray->priv->notify, NULL)) {
-		gpm_warning ("failed to send notification (%s)", content);
-		return FALSE;
-	}
-
-	return TRUE;
-}
-
-#elif (LIBNOTIFY_VERSION_MINOR == 2)
-
-static gboolean
-libnotify_event (GpmTrayIcon             *tray,
-		 guint                    timeout,	/* in seconds */
-		 const char              *subject,
-		 const char              *content,
-		 const LibNotifyEventType urgency)
-{
-	NotifyIcon  *icon = NULL;
-	NotifyHints *hints = NULL;
-	int          x;
-	int          y;
-
-	/* assertion checks */
-	g_assert (content);
-
-	hints = notify_hints_new ();
-	if (tray->priv->is_visible) {
-		get_widget_position (GTK_WIDGET (tray), &x, &y);
-		notify_hints_set_int (hints, "x", x);
-		notify_hints_set_int (hints, "y", y);
-	}
-	/* echo to terminal too */
-	if (urgency == LIBNOTIFY_URGENCY_CRITICAL) {
-		gpm_warning ("libnotify: %s : %s", GPM_NAME, content);
-	} else {
-		gpm_debug ("libnotify: %s : %s", GPM_NAME, content);
-	}
-
-	/* use default g-p-m icon for now */
-	icon = notify_icon_new_from_uri (GPM_DATA "gnome-power.png");
-
-	if (tray->priv->notify != NULL) {
-		notify_close (tray->priv->notify);
-	}
-
-	tray->priv->notify = notify_send_notification (tray->priv->notify,
-						       NULL,
-						       urgency,
-						       subject,
-						       content,
-						       icon, /* icon */
-						       TRUE,
-						       timeout,
-						       hints, /* hints */
-						       NULL, /* no user data */
-						       0);   /* no actions */
-	notify_icon_destroy (icon);
-
-	if (! tray->priv->notify) {
+	if (! notify_notification_show (icon->priv->notify, NULL)) {
 		gpm_warning ("failed to send notification (%s)", content);
 		return FALSE;
 	}
@@ -706,21 +818,30 @@ libnotify_event (GpmTrayIcon             *tray,
 
 #else
 
+/**
+ * libnotify_event:
+ * @icon: This icon class instance
+ * @title: The title, e.g. "Battery Low"
+ * @content: The contect, e.g. "17 minutes remaining"
+ * @timeout: The time we should remain on screen in seconds
+ * @msgicon: The icon to display, or NULL, e.g. GPM_STOCK_UPS_CHARGING_080
+ * @urgency: The urgency type, e.g. GPM_NOTIFY_URGENCY_CRITICAL
+ *
+ * Does a gtk messagebox dialogue.
+ * Return value: success
+ **/
 static gboolean
-libnotify_event (GpmTrayIcon             *tray,
-		 guint                    timeout,	/* in seconds */
-		 const char              *subject,
-		 const char              *content,
-		 const LibNotifyEventType urgency)
-
+libnotify_event (GpmTrayIcon    *icon,
+		 const char	*title,
+		 const char	*content,
+		 guint		 timeout,
+		 const char	*msgicon,
+		 GpmNotifyLevel	 urgency)
 {
 	GtkWidget     *dialog;
 	GtkMessageType msg_type;
 
-	/* assertion checks */
-	g_assert (content);
-
-	if (urgency == LIBNOTIFY_URGENCY_CRITICAL) {
+	if (urgency == GPM_NOTIFY_URGENCY_CRITICAL) {
 		msg_type = GTK_MESSAGE_WARNING;
 	} else {
 		msg_type = GTK_MESSAGE_INFO;
@@ -733,7 +854,7 @@ libnotify_event (GpmTrayIcon             *tray,
 						     "<span size='larger'><b>%s</b></span>",
 						     GPM_NAME);
 
-	gtk_message_dialog_format_secondary_markup (GTK_MESSAGE_DIALOG (dialog), content);
+	gtk_message_dialog_format_content_markup (GTK_MESSAGE_DIALOG (dialog), content);
 
 	g_signal_connect_swapped (dialog,
 				  "response",
@@ -746,46 +867,52 @@ libnotify_event (GpmTrayIcon             *tray,
 }
 #endif
 
+/**
+ * gpm_tray_icon_notify:
+ * @icon: This icon class instance
+ * @title: The title, e.g. "Battery Low"
+ * @content: The contect, e.g. "17 minutes remaining"
+ * @timeout: The time we should remain on screen in seconds
+ * @msgicon: The icon to display, or NULL, e.g. GPM_STOCK_UPS_CHARGING_080
+ * @urgency: The urgency type, e.g. GPM_NOTIFY_URGENCY_CRITICAL
+ *
+ * Does a libnotify or gtk messagebox dialogue.
+ **/
 void
-gpm_tray_icon_notify (GpmTrayIcon *icon,
-		      guint        timeout,	/* in seconds */
-		      const char  *primary,
-		      GtkWidget   *msgicon,
-		      const char  *secondary)
+gpm_tray_icon_notify (GpmTrayIcon	*icon,
+		      const char	*title,
+		      const char	*content,
+		      guint		 timeout,
+		      const char	*msgicon,
+		      GpmNotifyLevel	 urgency)
 {
 	g_return_if_fail (GPM_IS_TRAY_ICON (icon));
 
 	if (! icon->priv->show_notifications) {
-		gpm_debug ("ignoring notification: %s", primary);
+		gpm_debug ("ignoring notification: %s", title);
 		return;
 	}
 
-	gpm_debug ("doing notify: %s", primary);
-
-	libnotify_event (icon,
-			 timeout,
-			 primary,
-			 secondary,
-			 LIBNOTIFY_URGENCY_CRITICAL);
+	gpm_debug ("doing notify: %s", title);
+	libnotify_event (icon, title, content, timeout, msgicon, urgency);
 }
 
+/**
+ * gpm_tray_icon_cancel_notify:
+ * @icon: This icon class instance
+ *
+ * Cancels the notification, i.e. removes it from the screen.
+ **/
 void
 gpm_tray_icon_cancel_notify (GpmTrayIcon *icon)
 {
 	GError *error;
-
 	g_return_if_fail (GPM_IS_TRAY_ICON (icon));
-
 	error = NULL;
 
-#if LIBNOTIFY_VERSION_MINOR >= 2
+#ifdef HAVE_LIBNOTIFY
 	if (icon->priv->notify != NULL) {
-#if (LIBNOTIFY_VERSION_MINOR >= 3)
 		notify_notification_close (icon->priv->notify, &error);
-#elif (LIBNOTIFY_VERSION_MINOR == 2)
-		notify_close (icon->priv->notify);
-		icon->priv->notify = NULL;
-#endif
 	}
 	if (error != NULL) {
 		g_error_free (error);
